﻿using AlienRace;
using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Verse;
using Verse.Sound;
using BackstoryDef = RimWorld.BackstoryDef;

namespace ChangeRace
{
    public class ChangeRaceLogicDef : Def
    {
        public ChangeRace changeRace = new ChangeRace();
        public List<FactionOf> factionOf = new List<FactionOf>();
        public BackstoryLogic backstoryLogic = new BackstoryLogic();
        public List<Trait> defaultForbiddenTraits = new List<Trait>();

        public void ChangeRaceLogic(Pawn originalPawn, Hediff parentHediff)
        {
            TestErrorLog("1. Try check original pawn element");
            PawnProfile originalPawnProfile = new PawnProfile();
            InputPawnElement(originalPawn, originalPawnProfile);

            TestErrorLog("", "2. Try check condition for changeRace");
            var conditionAndResultList = changeRace.conditionAndResult;
            CheckConditionList(originalPawnProfile, conditionAndResultList, parentHediff);

        }

        private void CheckConditionList(PawnProfile originalPawnProfile, List<ConditionAndResult> conditionAndResultList, Hediff parentHediff)
        {
            Pawn pawn = originalPawnProfile.pawn;
            Map map = originalPawnProfile.map;
            IntVec3 position = originalPawnProfile.position;

            for (int i = 0; i < conditionAndResultList.Count; i++)
            {
                TestErrorLog($"Try ConditionAndResult List : {i}");
                ConditionAndResult conditionAndResult = conditionAndResultList[i];
                CheckConndition(originalPawnProfile, conditionAndResult.condition, out List<string> matchList, out int conditionListNum);


                if (matchList.Count == conditionListNum)
                {
                    TestErrorLog("", $"3. List : {i}'s All Condition Match. Start Change Race.");
                    List<ChangeRaceResult> changeRaceResultList = conditionAndResult.changeRaceResult;

                    Random rand = new Random();
                    for (int k = 0; k < changeRaceResultList.Count; k++)
                    {
                        int randChance = rand.Next(1, 101);
                        ChangeRaceResult changeRaceResult = changeRaceResultList[k];
                        if (randChance <= changeRaceResult.chance)
                        {
                            TestErrorLog($"The Chance : {changeRaceResult.chance} is higher than random number : {randChance}, so try start change race");
                            TestErrorLog($"ThingDef : {changeRaceResult.thingDef}");

                            //주변에 피 뿌리기
                            MakeFilth makeFilth = changeRaceResult.makeFilth;
                            CreateFilth(originalPawnProfile, makeFilth.filthDef, makeFilth.filthNum);

                            //소리
                            if (changeRaceResult.soundDef != null)
                            {
                                SoundStarter.PlayOneShot(changeRaceResult.soundDef, SoundInfo.InMap(pawn, 0));
                            }

                            DropItem(originalPawnProfile, changeRaceResult);

                            //만약 thingDef가 아이템 계열이 아닌 경우
                            if (!ChangeRaceResultIsItem(originalPawnProfile, parentHediff, changeRaceResult))
                            {
                                CreateReplacedPawn(
                                    originalPawnProfile: originalPawnProfile,
                                    changeRaceResult: changeRaceResult,
                                    factionOf: factionOf,
                                    parentHediff: parentHediff,
                                    generatedPawn: out Pawn generatedPawn
                                    );

                                if (generatedPawn == null)
                                {
                                    //야생 폰인데, 우회용 폰카인드등이 없을경우 종료
                                    return;
                                }

                                TestErrorLog("Try spawn to original pawn position");

                                GenSpawn.Spawn(generatedPawn, position, map, 0);

                                TestErrorLog("generatedPawn.def : " + generatedPawn.def.defName);

                                TestErrorLog("$$$$ Finish Spawn $$$$");

                                pawn.Destroy(DestroyMode.Vanish);
                                return;
                            }

                        }
                        else
                        {
                            TestErrorLog($"The Chance : {changeRaceResult.chance} is lower than random number : {randChance}, so try the next list");
                        }
                    }
                }
                if (i == conditionAndResultList.Count - 1)
                {
                    TestErrorLog("No more conditionAndResultList, finish Logic.");
                    return;
                }
            }
        }

        private void CreateReplacedPawn(PawnProfile originalPawnProfile, ChangeRaceResult changeRaceResult, List<FactionOf> factionOf, Hediff parentHediff, out Pawn generatedPawn)
        {
            Pawn originalPawn = originalPawnProfile.pawn;
            TestErrorLog("Create Replaced Pawn");
            //성별
            Gender? generatedPawnGender = SetGender(originalPawnProfile, changeRaceResult);
            //팩션, 폰카인드, 와일드맨 여부
            (Faction generatedPawnFaction, PawnKindDef generatedPawnKindDef, bool isWildMan) = SetFaction_PawnKindDef(originalPawnProfile, changeRaceResult, factionOf);

            PawnGenerationRequest generationRequest = new PawnGenerationRequest(
            kind: generatedPawnKindDef,
            faction: generatedPawnFaction,
            fixedBiologicalAge: originalPawnProfile.ageBiological,
            fixedChronologicalAge: originalPawnProfile.ageChronological,
            fixedGender: generatedPawnGender);

            TestErrorLog("Try generate Pawn : ");
            TestErrorLog($"Faction : {generatedPawnFaction?.ToString() ?? "null"}");
            TestErrorLog($"PawnKindDef : {changeRaceResult.pawnKindDef.defName}");
            TestErrorLog($"AgeBiologicalYearsFloat : {originalPawnProfile.ageBiological}");
            TestErrorLog($"Gender : {generatedPawnGender}");

            TestErrorLog("Generate pawn and create");
            // generate pawn and create 

            generatedPawn = PawnGenerator.GeneratePawn(generationRequest);
            TestErrorLog($"Gender : {generatedPawnGender}");

            generatedPawn.ageTracker.AgeBiologicalTicks = originalPawnProfile.ageBiological;
            generatedPawn.ageTracker.AgeChronologicalTicks = originalPawnProfile.ageChronological;

            //헤디프 추가
            TestErrorLog("Generate pawn add hediff : ");
            List<HediffDef> addHediffDef = changeRaceResult.addHediffDef;
            for (int k = 0; k < addHediffDef.Count; k++)
            {
                generatedPawn.health.AddHediff(addHediffDef[k]);

                TestErrorLog($" * {addHediffDef[k].defName}");
            }

            if (isWildMan && generatedPawn.kindDef.race.race.Humanlike)
            {
                generatedPawn.kindDef = PawnKindDefOf.WildMan;
            }

            if (changeRaceResult.thingDef is ThingDef_AlienRace alienThing && alienThing.alienRace.generalSettings.maleGenderProbability < 0.001)
            {
                Log.Error($"Because {alienThing.defName}'s maleGenderProbability is low then 0.001, so copy names may fail.");
            }

            CopyName(originalPawnProfile.pawn, generatedPawn);
            TestErrorLog($"Copy pawn name : {generatedPawn.Name}");

            // 관계가 이어짐 (20201112 확인)
            if (changeRaceResult.keepWhat.keepReration)
            {
                TranslatePawnRelations(originalPawn, generatedPawn, changeRaceResult.bondChangeToLover);
                TestErrorLog("Relation Setting :  Keep reration");
            }
            else { TestErrorLog("Relation Setting : Delete reration"); }

            //사회 무드가 이어짐
            if (originalPawn.RaceProps.Humanlike && changeRaceResult.keepWhat.keepThought_MemorySocial)
            {
                TranslatePawnThought_MemorySocial(originalPawn, generatedPawn);
                TestErrorLog("Thought_MemorySocial : Keep");
            }
            else { TestErrorLog("Thought_MemorySocial : Delete"); }


            TestErrorLog("", "5. Log Generated Pawn Profile");
            PawnProfile generatedPawnProfile = new PawnProfile();
            InputPawnElement(generatedPawn, generatedPawnProfile);

            if (generatedPawn.RaceProps.Humanlike)
            {
                //이하 인간 계열
                //백스 설정
                (var childhood, var adult) = GetBackstory(originalPawnProfile, generatedPawnProfile, backstoryLogic);

                generatedPawn.story.Childhood = childhood;
                generatedPawn.story.Adulthood = adult;

                //외형
                TestErrorLog("", "7. Set appearance");
                Random rand = new Random();
                Random rand2 = new Random();
                List<Appearance> appearanceList = changeRaceResult.appearance;
                for (int k = 0; k < appearanceList.Count; k++)
                {
                    //먼저 확률부터 계산
                    int randNum = rand.Next(1, 101);
                    Appearance appearance = appearanceList[k];
                    if (randNum <= appearance.chance)
                    {
                        TestErrorLog($" * Try use appearance list : {k}");
                        if (appearance.headTypeDef != null)
                        {
                            generatedPawn.story.headType = appearance.headTypeDef;
                            TestErrorLog($" * Head : ", appearance.headTypeDef.defName);
                        }
                        if (appearance.hairDef != null)
                        {
                            generatedPawn.story.hairDef = appearance.hairDef;
                            TestErrorLog(" * HairDef : ", appearance.hairDef.defName);
                        }
                        if (appearance.bodyTypeDef != null)
                        {
                            generatedPawn.story.bodyType = appearance.bodyTypeDef;
                            TestErrorLog(" * BodyTypeDef : ", appearance.bodyTypeDef.defName);
                        }

                        //헤어색
                        if (changeRaceResult.keepWhat.keepHairColor && originalPawn.story.HairColor != null)
                        {
                            TestErrorLog($" * Hair color : keep original pawn hair color");
                            generatedPawn.story.HairColor = originalPawn.story.HairColor;

                            //TestErrorLog("fail to keep original pawn hair color, It could be because the original pawn is not humanlike or does not have a Hair def.");
                        }
                        else if (!appearance.hairColor.NullOrEmpty())
                        {
                            int randNum2 = rand2.Next(0, appearance.hairColor.Count);
                            generatedPawn.story.HairColor = appearance.hairColor[randNum2];
                            TestErrorLog($" * Hair color : {generatedPawn.story.HairColor} (List : {randNum2})");
                        }
                        break;
                    }
                }

                //스킬 복사
                TestErrorLog("", "8. Set Skill, Trait");
                TestErrorLog("Copy or make skills");
                generatedPawn.skills = originalPawn.skills ?? generatedPawn.skills; // move skills

                //트레잇 설정
                TestErrorLog(" * Set trait");
                //지금 여기 문제, 아마 null 처리가 제대로 안되는듯
                if (originalPawn.RaceProps.Humanlike)
                {
                    generatedPawn.story.traits.allTraits = originalPawn.story.traits.allTraits ?? generatedPawn.story.traits.allTraits; // move traits
                }
                TestErrorLog(" * Remove forbidden trait");
                var forbiddenTraitsList = changeRaceResult.forbiddenTraits;
                if (changeRaceResult.useDefaultForbiddenTraits)
                {
                    forbiddenTraitsList = defaultForbiddenTraits;
                }
                RemoveForbiddenTrait(ref generatedPawn, forbiddenTraitsList);
            }
            SendMessage(originalPawn, generatedPawn, parentHediff, changeRaceResult);
        }

        private void RemoveForbiddenTrait(ref Pawn pawn, List<Trait> forbiddenTraits)
        {
            var pawnTraits = pawn.story.traits.allTraits;

            for (int i = 0; i < forbiddenTraits.Count; i++)
            {
                Trait forbiddenTrait = forbiddenTraits[i];
                for (int j = 0; j < pawnTraits.Count; j++)
                {
                    if (pawnTraits[j].def == forbiddenTrait.def && pawnTraits[j].Degree == forbiddenTrait.Degree)
                    {
                        TestErrorLog("=================");
                        TestErrorLog(pawnTraits[j].def.defName);
                        TestErrorLog(pawnTraits[j].Degree.ToString());

                        pawnTraits.Remove(pawnTraits[j]);
                    }
                }
            }
            TestErrorLog("=================");
        }

        private (BackstoryDef childhood, BackstoryDef adult) GetBackstory(PawnProfile originalPawnProfile, PawnProfile generatedPawnProfile, BackstoryLogic backstoryLogic)
        {
            BackstoryDef childHoodBackstory = null;
            BackstoryDef adultHoodBackstory = null;
            bool useOriginalBackstory = false;
            string childHoodBackstoryString = "";
            List<string> childHoodBackstoryStringList = new List<string>();

            //백스 설정
            BuildString_ChildHoodBackstoryStringList(originalPawnProfile, generatedPawnProfile, backstoryLogic, ref useOriginalBackstory, ref childHoodBackstoryString, childHoodBackstoryStringList);


            TestErrorLog("Try Set ChildHoodBackstory : ");

            if (!useOriginalBackstory)
            {
                bool tryDefaultBackstory = true;
                for (int i = childHoodBackstoryStringList.Count - 1; 0 <= i; i--)
                {
                    if (DefDatabase<BackstoryDef>.GetNamed(childHoodBackstoryStringList[i],false) != null)
                    {
                        childHoodBackstory = DefDatabase<BackstoryDef>.GetNamed(childHoodBackstoryStringList[i]);
                        tryDefaultBackstory = false;
                        TestErrorLog($"Success : ChildHoodBackstoryDef : {childHoodBackstoryStringList[i]}");
                        break;
                    }
                    else
                    {
                        TestErrorLog($"{childHoodBackstoryStringList[i]} is null childHoodBackstoryDef. Try Next...");
                    }
                }
                if (tryDefaultBackstory)
                {
                    TestErrorLog("All custom backstory setting is failed, Try set defaultBackstoryDef");
                    childHoodBackstory = backstoryLogic.defaultBackstoryDef;
                    adultHoodBackstory = null;
                    TestErrorLog("childHoodBackstory : ", childHoodBackstory.ToString(), "adultHoodBackstory : ", "null");
                }
            }
            else if (useOriginalBackstory || childHoodBackstory == null)
            {
                TestErrorLog("Keep original backstory");
                childHoodBackstory = originalPawnProfile.childhood;
                adultHoodBackstory = originalPawnProfile.adulthood;
                TestErrorLog("childHoodBackstory : ", childHoodBackstory.ToString() ?? "null", "adultHoodBackstory : ", adultHoodBackstory.ToString() ?? "null");
            }
            return (childHoodBackstory, adultHoodBackstory);
        }

        private void BuildString_ChildHoodBackstoryStringList(PawnProfile originalPawnProfile, PawnProfile generatedPawnProfile, BackstoryLogic backstoryLogic, ref bool useOriginalBackstory, ref string childHoodBackstoryString, List<string> childHoodBackstoryStringList)
        {
            if (!backstoryLogic.keepOriginalPawnBackstory)
            {
                TestErrorLog("", "6. Out Element Name");
                //현재 종족, 접두사
                CustomElementName customElementName = backstoryLogic.customElementName;
                DefaultElementName defaultElementName = backstoryLogic.defaultElementName;
                DefaultElementName tempElementName = backstoryLogic.tempElementName;

                var prefixElement = customElementName.prefixElement;
                OutElementName(generatedPawnProfile,
                               generatedPawnProfile.thingDef,
                               prefixElement,
                               "Prefix",
                               defaultElementName.prefix,
                               ref tempElementName.prefix,
                               ref useOriginalBackstory);

                //이전 종족
                var raceElement = customElementName.raceElement;
                OutElementName(originalPawnProfile,
                               originalPawnProfile.thingDef,
                               raceElement,
                               "Race",
                               defaultElementName.race,
                               ref tempElementName.race,
                               ref useOriginalBackstory);

                //이전 관계
                var factionOf_BeforeElement = customElementName.factionOf_BeforeElement;
                OutElementName(originalPawnProfile,
                               originalPawnProfile.factionOf,
                               factionOf_BeforeElement,
                               "factionOf_Before",
                               defaultElementName.factionOf_Before,
                               ref tempElementName.factionOf_Before,
                               ref useOriginalBackstory);

                //현재 관계
                var factionOf_AfterElement = customElementName.factionOf_AfterElement;
                OutElementName(generatedPawnProfile,
                               generatedPawnProfile.factionOf,
                               factionOf_AfterElement,
                               "factionOf_After",
                               defaultElementName.factionOf_After,
                               ref tempElementName.factionOf_After,
                               ref useOriginalBackstory);

                //로얄 타이틀
                var royalTitleElement = customElementName.royalTitleElement;
                OutElementName(originalPawnProfile,
                               originalPawnProfile.royalTitleDef,
                               royalTitleElement,
                               "RoyalTitle",
                               defaultElementName.royalTitle,
                               ref tempElementName.royalTitle,
                               ref useOriginalBackstory);

                //팩션Def이전
                var faction_BeforeElement = customElementName.faction_BeforeElement;
                OutElementName(originalPawnProfile,
                               originalPawnProfile.factionDef,
                               faction_BeforeElement,
                               "Faction",
                               defaultElementName.faction_Before,
                               ref tempElementName.faction_Before,
                               ref useOriginalBackstory);

                //팩션Def이후
                var faction_AfterElement = customElementName.faction_AfterElement;
                OutElementName(generatedPawnProfile,
                               generatedPawnProfile.factionDef,
                               faction_AfterElement,
                               "Faction",
                               defaultElementName.faction_After,
                               ref tempElementName.faction_After,
                               ref useOriginalBackstory);

                //동물훈련
                var trainabilityElement = customElementName.trainabilityElement;
                OutElementName(originalPawnProfile,
                               originalPawnProfile.trainabilityDef,
                               trainabilityElement,
                               "Trainability",
                               defaultElementName.trainability,
                               ref tempElementName.trainability,
                               ref useOriginalBackstory);

                //지능
                var intelligenceElement = customElementName.intelligenceElement;
                OutElementName(originalPawnProfile,
                               originalPawnProfile.intelligence,
                               intelligenceElement,
                               "Intelligence",
                               defaultElementName.intelligence,
                               ref tempElementName.intelligence,
                               ref useOriginalBackstory);

                //이전 성별
                var gender_BeforeElement = customElementName.gender_BeforeElement;
                OutElementName(originalPawnProfile,
                               originalPawnProfile.gender,
                               gender_BeforeElement,
                               "Gender_Before",
                               defaultElementName.gender_Before,
                               ref tempElementName.gender_Before,
                               ref useOriginalBackstory);

                //현재 성별
                var gender_AfterElement = customElementName.gender_AfterElement;
                OutElementName(generatedPawnProfile,
                               generatedPawnProfile.gender,
                               gender_AfterElement,
                               "Gender_After",
                               defaultElementName.gender_After,
                               ref tempElementName.gender_After,
                               ref useOriginalBackstory);

                //나이
                var ageElement = customElementName.ageElement;
                tempElementName.age = defaultElementName.age;
                for (int i = 0; i < ageElement.Count; i++)
                {
                    List<Age> element = ageElement[i].element;
                    bool Humanlike = ageElement[i].humanlike && originalPawnProfile.intelligence == Intelligence.Humanlike;
                    bool NonHumanlike = ageElement[i].nonHumanlike && originalPawnProfile.intelligence != Intelligence.Humanlike;
                    bool ForBoth = !ageElement[i].humanlike && !ageElement[i].nonHumanlike;
                    if (Humanlike || NonHumanlike || ForBoth)
                    {
                        for (int j = 0; j < element.Count; j++)
                        {
                            if (element[j].min <= originalPawnProfile.ageBiological && originalPawnProfile.ageBiological <= element[j].max && !useOriginalBackstory)
                            {
                                if (ageElement[i].keepOriginalBackstory)
                                {
                                    useOriginalBackstory = true;
                                }
                                else
                                {
                                    tempElementName.age = ageElement[i].elementName;
                                }
                                break;
                            }
                        }
                    }
                }
                TestErrorLog($" * Age : {tempElementName.age}");

                //이전 폰카인드
                var pawnKind_BeforeElement = customElementName.pawnKind_BeforeElement;
                OutElementName(originalPawnProfile,
                               originalPawnProfile.pawnKindDef,
                               pawnKind_BeforeElement,
                               "PawnKind_Before",
                               defaultElementName.pawnKind_Before,
                               ref tempElementName.pawnKind_Before,
                               ref useOriginalBackstory);

                //현재 폰카인드
                var pawnKind_AfterElement = customElementName.pawnKind_AfterElement;
                OutElementName(generatedPawnProfile,
                               generatedPawnProfile.pawnKindDef,
                               pawnKind_AfterElement,
                               "PawnKind_After",
                               defaultElementName.pawnKind_After,
                               ref tempElementName.pawnKind_After,
                               ref useOriginalBackstory);

                //팩션 FactionRelationKind 이전
                var factionRelationKind_BeforeElement = customElementName.factionRelationKind_BeforeElement;
                OutElementName(originalPawnProfile,
                               originalPawnProfile.factionRelationKind,
                               factionRelationKind_BeforeElement,
                               "FactionRelationKind_Before",
                               defaultElementName.factionRelationKind_Before,
                               ref tempElementName.factionRelationKind_Before,
                               ref useOriginalBackstory);

                //팩션 FactionRelationKind 이후
                var factionRelationKind_AfterElement = customElementName.factionRelationKind_AfterElement;
                OutElementName(generatedPawnProfile,
                               generatedPawnProfile.factionRelationKind,
                               factionRelationKind_AfterElement,
                               "factionRelationKind_After",
                               defaultElementName.factionRelationKind_After,
                               ref tempElementName.factionRelationKind_After,
                               ref useOriginalBackstory);

                //콜로니스트, 노예, 수감자 이전
                var pawnIs_BeforeElement = customElementName.pawnIs_BeforeElement;
                OutElementName(originalPawnProfile,
                               originalPawnProfile.pawnIs,
                               pawnIs_BeforeElement,
                               "pawnIs_Before",
                               defaultElementName.pawnIs_Before,
                               ref tempElementName.pawnIs_Before,
                               ref useOriginalBackstory);

                //콜로니스트, 노예, 수감자 이후
                var pawnIs_AfterElement = customElementName.pawnIs_AfterElement;
                OutElementName(generatedPawnProfile,
                               generatedPawnProfile.pawnIs,
                               pawnIs_AfterElement,
                               "pawnIs_After",
                               defaultElementName.pawnIs_After,
                               ref tempElementName.pawnIs_After,
                               ref useOriginalBackstory);

                //마지막으로 스트링 합치기
                for (int i = 0; i < backstoryLogic.backstoryElementSequence.Count; i++)
                {
                    try
                    {
                        Type tp = typeof(DefaultElementName);
                        FieldInfo fld = tp.GetField(backstoryLogic.backstoryElementSequence[i], BindingFlags.Instance |
                                               BindingFlags.NonPublic |
                                               BindingFlags.Public |
                                               BindingFlags.NonPublic);
                        object point = fld.GetValue(tempElementName);
                        if ((string)point != "")
                        {
                            childHoodBackstoryString += (string)point;
                            childHoodBackstoryStringList.Add(childHoodBackstoryString);
                        }
                    }
                    catch
                    {
                        Log.Error(backstoryLogic.backstoryElementSequence[i] + " is null backstoryElement");
                    }
                }
                TestErrorLog($"Assembled childHoodBackstory string : {childHoodBackstoryString}");
            }
        }

        private void OutElementName<T>(PawnProfile pawnProfile, T originalPawnCondition, List<BackstoryElement<T>> backstoryElementList, string elementType, string defaultElementName, ref string tempElementName, ref bool useOriginalBackstory)
        {
            tempElementName = defaultElementName;
            if (!useOriginalBackstory)
            {
                for (int i = 0; i < backstoryElementList.Count; i++)
                {
                    if (originalPawnCondition != null)
                    {
                        BackstoryElement<T> backstoryElement = backstoryElementList[i];
                        bool Humanlike = backstoryElement.humanlike && pawnProfile.intelligence == Intelligence.Humanlike;
                        bool NonHumanlike = backstoryElement.nonHumanlike && pawnProfile.intelligence != Intelligence.Humanlike;
                        bool ForBoth = backstoryElement.humanlike && backstoryElement.nonHumanlike;
                        if (Humanlike || NonHumanlike || ForBoth)
                        {
                            for (int j = 0; j < backstoryElement.element.Count; j++)
                            {
                                if (useOriginalBackstory)
                                {
                                    break;
                                }

                                else if ((bool)(originalPawnCondition?.Equals(backstoryElement.element[j])))
                                {
                                    if (backstoryElement.keepOriginalBackstory)
                                    {
                                        useOriginalBackstory = true;
                                    }
                                    else
                                    {
                                        tempElementName = backstoryElement.elementName;
                                    }
                                    break;
                                }

                            }
                        }
                    }
                }
                TestErrorLog($" * {elementType} : {tempElementName}");
            }
        }


        private void TranslatePawnThought_MemorySocial(Pawn from, Pawn to)
        {
            var thought_MemorySocial_Mood_DefName = "AAA123";
            for (int i = 0; i < from.needs.mood.thoughts.memories.Memories.Count; i++)
            {
                var thought_MemorySocial = from.needs.mood.thoughts.memories.Memories[i];
                if (thought_MemorySocial is Thought_MemorySocial || thought_MemorySocial.def.defName.Contains(thought_MemorySocial_Mood_DefName))
                {
                    to.needs.mood.thoughts.memories.Memories.Add(thought_MemorySocial);
                    thought_MemorySocial_Mood_DefName = thought_MemorySocial.def.defName;
                }
            }
        }

        private void TranslatePawnRelations(Pawn from, Pawn to, bool bondChangeToLover)
        {
            for (int i = 0; i < from.relations.DirectRelations.Count; i++)
            {
                DirectPawnRelation relation = from.relations.DirectRelations[i];
                if (relation.def == PawnRelationDefOf.Bond && bondChangeToLover)
                {
                    relation.def = PawnRelationDefOf.Lover;
                }
                else
                {
                    RemoveRelationFromOtherRelatedPawns(from);
                }
                to.relations.AddDirectRelation(relation.def, relation.otherPawn);
            }
            RemoveRelationFromOtherRelatedPawns(from);
        }
        private void RemoveRelationFromOtherRelatedPawns(Pawn pawn)
        {
            // pawn과 관계가 있는 모든 폰에게서 pawn에 관한 관계를 삭제
            var relatedPawns = pawn.relations.RelatedPawns;
            foreach (var otherPawn in relatedPawns)
            {
                var relationsList = new List<DirectPawnRelation>(otherPawn.relations.DirectRelations);
                foreach (var relation in relationsList.Where(r => r.otherPawn == pawn))
                    otherPawn.relations.RemoveDirectRelation(relation);
            }
        }

        private void CopyName(Pawn from, Pawn to)
        {
            if (from.Name != null)
            {
                var name = from.Name;
                if (name is NameTriple triple)
                    to.Name = new NameTriple(triple.First, triple.Nick, triple.Last);
                else if (name is NameSingle single)
                    to.Name = new NameSingle(single.Name, single.Numerical);
                else
                    to.Name = name;
            }
        }

        private (Faction generatedPawnFaction, PawnKindDef generatedPawnKindDef, bool isWildMan) SetFaction_PawnKindDef(PawnProfile pawnProfile, ChangeRaceResult changeRaceResult, List<FactionOf> factionOf)
        {
            TestErrorLog("", "4. Try set pawnKind and faction");
            PawnKindDef kindDef = changeRaceResult.pawnKindDef;
            Faction faction = null;
            bool isWildMan = false;

            if (changeRaceResult.pawnKindDef == PawnKindDefOf.WildMan)
            {
                TestErrorLog("Set to Wild (pawnkind)");
                 MakeWildMan(changeRaceResult, out kindDef, out faction, out isWildMan);

            }
            else if (changeRaceResult.makeOfPlayer)
            {
                TestErrorLog("Set to OfPlayer (makeOfPlayer)");
                faction = Faction.OfPlayer;
            }
            else if (changeRaceResult.keepWhat.keepFaction)
            {
                TestErrorLog("Keep original pawn faction (keepFaction)");
                faction = pawnProfile.faction;
            }
            else
            {
                for (int i = 0; i < factionOf.Count; i++)
                {
                    //조건 체크, 만약 조건리스트중 하나의 조건이라도 만족하면 다음 조건을 체크함
                    TestErrorLog($"Try FactionOf List : {i}");
                    CheckConndition(pawnProfile, factionOf[i].condition, out List<string> matchList, out int conditionListNum);

                    if (matchList.Count == conditionListNum)
                    {
                        TestErrorLog("", $"4.1. List : {i}'s All Condition Match. Start Set Faction.");
                        FactionResult factionResult = factionOf[i].factionResult;
                        if (factionResult.keepOriginalFaction)
                        {
                            TestErrorLog("Keep original pawn faction (condition)");
                            faction = pawnProfile.faction;
                            break;
                        }
                        else if (factionResult.isPlayerFaction)
                        {
                            TestErrorLog("Set to OfPlayer (condition)");
                            faction = Faction.OfPlayer;
                            break;
                        }
                        else if (factionResult.isWildMan)
                        {
                            TestErrorLog("Set to Wild (condition)");
                            MakeWildMan(changeRaceResult, out kindDef, out faction, out isWildMan);
                        }
                    }
                    else if (i == changeRace.conditionAndResult.Count - 1)
                    {
                        TestErrorLog("All of FactionResult are false : try set faction use factionOf");
                        if (pawnProfile.factionOf != "OfUnkwon")
                        {
                            faction = (Faction)typeof(Faction).GetProperty(pawnProfile.factionOf, BindingFlags.Public | BindingFlags.Static).GetValue(null);
                            break;
                        }
                        else
                        {
                            TestErrorLog("Fail use factionOf (condition unmatch), Set to Null Faction");
                            faction = null;
                            break;
                        }
                    }
                }
            }

            if (!isWildMan)
            {
                TestErrorLog($"generatedPawn pawnKindDef : {kindDef.defName}");
            }
            else if (isWildMan && !kindDef.race.race.Humanlike)
            {
                TestErrorLog($"generatedPawn pawnKindDef : {kindDef.defName}");
            }
            else
            {
                TestErrorLog($"temporaryPawnKindDefForWildMan : {kindDef.defName}");
                TestErrorLog($"generatedPawn pawnKindDef : {PawnKindDefOf.WildMan.defName}");
            }

            return (faction, kindDef, isWildMan);
        }

        private void MakeWildMan(ChangeRaceResult changeRaceResult, out PawnKindDef kindDef, out Faction faction, out bool isWildMan)
        {
            kindDef = new PawnKindDef();
            TestErrorLog($"ThingDef : {changeRaceResult.thingDef}");
            isWildMan = true;
            faction = null;
            if(changeRaceResult.pawnKindDef != PawnKindDefOf.WildMan)
            {
                kindDef = changeRaceResult.pawnKindDef;
            }
            else if (changeRaceResult.temporaryPawnKindDefForWildMan == null)
            {
                foreach (var temporaryPawnKindDef in DefDatabase<PawnKindDef>.AllDefs)
                {
                    if (temporaryPawnKindDef.race == changeRaceResult.thingDef)
                    {
                        kindDef = temporaryPawnKindDef;
                    }
                    break;
                }
            }
            else kindDef = changeRaceResult.temporaryPawnKindDefForWildMan;
        }

        private Gender? SetGender(PawnProfile pawnProfile, ChangeRaceResult changeRaceResult)
        {
            TestErrorLog($"Set generated pawn Gender");
            Gender? generatedPawnGender = null;
            Random randGender1 = new Random();
            int randGenderNum1 = randGender1.Next(1, 101);
            Random randGender2 = new Random();
            int randGenderNum2 = randGender2.Next(1, 101);

            //성별 설정
            if (!changeRaceResult.thingDef.race.hasGenders)
            {
                TestErrorLog($"{changeRaceResult.thingDef.race}'s gender is none");
                generatedPawnGender = Gender.None;
            }
            else if (changeRaceResult.keepWhat.keepGender)
            {
                TestErrorLog($"keepGender is true : generatedPawnGender = {pawnProfile.gender}");
                generatedPawnGender = pawnProfile.gender;
            }
            else if (randGenderNum1 <= changeRaceResult.genderResult.Female)
            {
                generatedPawnGender = Gender.Female;
                TestErrorLog($"keepGender is false : Try draw generatedPawnGender = {generatedPawnGender}");
            }
            else if (randGenderNum2 <= changeRaceResult.genderResult.Male)
            {
                generatedPawnGender = Gender.Male;
                TestErrorLog($"keepGender is false : Try draw generatedPawnGender = {generatedPawnGender}");
            }
            else
            {
                TestErrorLog($"keepGender is false : Set None = random set gender");
            }
            return generatedPawnGender;
        }

        private bool ChangeRaceResultIsItem(PawnProfile originalPawnProfile, Hediff parentHediff, ChangeRaceResult changeRaceResult)
        {
            if (changeRaceResult.thingDef.category == ThingCategory.Item)
            {
                TestErrorLog($"$$$${changeRaceResult.thingDef.defName} is Item category $$$$");
                Thing thing = ThingMaker.MakeThing(changeRaceResult.thingDef, null);
                thing.stackCount = changeRaceResult.stack;
                GenPlace.TryPlaceThing(thing, originalPawnProfile.position, originalPawnProfile.map, 0, null, null, default);
                originalPawnProfile.pawn.Destroy(DestroyMode.Vanish);
                TestErrorLog("$$$$ Finish Change $$$$");
                SendMessage(originalPawnProfile.pawn, thing, parentHediff, changeRaceResult);

                return true;
            }
            return false;
        }

        private void SendMessage(Pawn originalPawn, Thing generatedThing, Hediff parentHediff, ChangeRaceResult changeRaceResult)
        {
            var generatedThingLabel = generatedThing.def.label;
            if (generatedThing.def is ThingDef_AlienRace alienGeneratedThing)
            {
                generatedThingLabel = alienGeneratedThing.label;
            }
            if (changeRaceResult.sendMessageOrLetter.message != null)
            {
                Messages.Message(
                    changeRaceResult.sendMessageOrLetter.message.Translate(originalPawn.LabelShort, generatedThingLabel, parentHediff.def.defName.CapitalizeFirst()).CapitalizeFirst(),
                    generatedThing, changeRaceResult.sendMessageOrLetter.messageTypeDef ?? MessageTypeDefOf.NegativeHealthEvent,
                    true);
            }
            if (changeRaceResult.sendMessageOrLetter.letterLabel != null && changeRaceResult.sendMessageOrLetter.letterText != null)
            {
                Find.LetterStack.ReceiveLetter(
                    changeRaceResult.sendMessageOrLetter.letterLabel.Translate(originalPawn.LabelShort, generatedThingLabel, parentHediff.def.defName.CapitalizeFirst()).CapitalizeFirst(),
                    changeRaceResult.sendMessageOrLetter.letterText.Translate(originalPawn.LabelShort, generatedThingLabel, parentHediff.def.defName.CapitalizeFirst()).CapitalizeFirst(),
                    changeRaceResult.sendMessageOrLetter.letterDef ?? LetterDefOf.NegativeEvent,
                    generatedThing,
                    null, null, null, null); ;
            }
        }

        private void DropItem(PawnProfile pawnProfile, ChangeRaceResult changeRaceResult)
        {
            //아군의 장비 떨굼
            Faction faction = pawnProfile.faction;
            Pawn pawn = pawnProfile.pawn;
            bool playerSide = (pawn.IsPrisonerOfColony || pawn.IsFreeColonist || pawn.IsColonist || pawn.IsSlaveOfColony);

            if (faction == Faction.OfPlayer || faction == Faction.OfPlayerSilentFail || changeRaceResult.unOfColonyPawnOrAnimalDropItem)
            {
                TestErrorLog("Drop Item");
                //동물 및 폰 아이템 떨구기
                pawn.DropAndForbidEverything();
                //아군 폰 계열이거나, 플래그가 true 이고, 다른세력일 경우
                if ((playerSide || changeRaceResult.unOfColonyPawnOrAnimalDropItem) && pawnProfile.intelligence == Intelligence.Humanlike)
                {
                    TestErrorLog("Drop apparel and equipment");
                    pawn.apparel.DropAll(pawn.Position, true, true);
                    pawn.equipment.DropAllEquipment(pawn.Position, true);
                }
            }
        }

        private void CreateFilth(PawnProfile pawnProfile, ThingDef filthDef, int filthNum)
        {
            TestErrorLog($"FilthDef : {filthDef?.defName ?? "null"}");
            if (filthDef != null)
            {
                for (int i = 0; i < filthNum; i++)
                {
                    IntVec3 pawnPosition = pawnProfile.position;
                    IntVec3 Pos = pawnPosition + GenRadial.RadialPattern[i];
                    Map pawnMap = pawnProfile.map;
                    Pawn pawn = pawnProfile.pawn;

                    bool canPlaceFilthToPos = GenGrid.InBounds(Pos, pawnMap) && GridsUtility.GetRoomOrAdjacent(pawnPosition, pawnMap, (RegionType)6) == GridsUtility.GetRoomOrAdjacent(Pos, pawnMap, (RegionType)6);
                    if (canPlaceFilthToPos)
                    {
                        FilthMaker.TryMakeFilth(pawnPosition, pawnMap, filthDef, GenText.LabelIndefinite(pawn), Rand.RangeInclusive(0, 4));
                    }
                }
            }
        }

        private void CheckConndition(PawnProfile originalPawnProfile, List<Condition> conditions, out List<string> matchList, out int conditionListNum)
        {
            conditionListNum = conditions.Count;
            matchList = new List<string>();
            List<string> unMatchList = new List<string>();

            for (int i = 0; i < conditions.Count; i++)
            {
                var condition = conditions[i];
                //종족
                ConditionMatch(
                    originalPawnProfile.thingDef,
                    condition.thingDef,
                    matchList, unMatchList);

                //훈련
                ConditionMatch(
                    originalPawnProfile.trainabilityDef,
                    condition.trainabilityDef,
                    matchList, unMatchList);

                //지능
                ConditionMatch(
                    originalPawnProfile.intelligence,
                    condition.intelligence,
                    matchList, unMatchList);

                //폰카인드
                ConditionMatch(
                    originalPawnProfile.pawnKindDef,
                    condition.pawnKindDef,
                    matchList, unMatchList);

                //로얄티
                ConditionMatch(
                    originalPawnProfile.royalTitleDef,
                    condition.royalTitleDef,
                    matchList, unMatchList);

                //성별
                ConditionMatch(
                    originalPawnProfile.gender,
                    condition.gender,
                    matchList, unMatchList);

                //팩션Def
                ConditionMatch(
                    originalPawnProfile.factionDef,
                    condition.factionDef,
                    matchList, unMatchList);

                //팩션관계
                ConditionMatch(
                    originalPawnProfile.factionRelationKind,
                    condition.factionRelationKind,
                    matchList, unMatchList);

                //팩션관계
                ConditionMatch(
                    originalPawnProfile.factionOf,
                    condition.factionOf,
                    matchList, unMatchList);

                //폰Is
                ConditionMatch(
                    originalPawnProfile.pawnIs,
                    condition.pawnIs,
                    matchList, unMatchList);
            }
            for (int j = 0; j < matchList.Count; j++)
            {
                TestErrorLog(matchList[j]);
            }
            for (int k = 0; k < unMatchList.Count; k++)
            {
                TestErrorLog(unMatchList[k]);
            }
        }

        private static void ConditionMatch<T>(T pawnProfile, List<T> conditionElement, List<string> matchList, List<string> unMatchList)
        {
            for (int i = 0; i < conditionElement.Count; i++)
            {
                if (conditionElement[i].Equals(pawnProfile))
                {
                    matchList.Add($" * Match : {conditionElement[i].GetType()} : {conditionElement[i]}");
                    break;
                }
                else if (i == conditionElement.Count - 1)
                {
                    string unMatchConditionElement = "";
                    for (int k = 0; k < conditionElement.Count; k++)
                    {
                        unMatchConditionElement += conditionElement[k].ToString() + ", ";
                    }
                    unMatchList.Add($" * Unmatch : {conditionElement[i].GetType()} : {unMatchConditionElement.Remove(unMatchConditionElement.Length - 2)}");
                };
            }
        }

        private void InputPawnElement(Pawn input, PawnProfile pawnProfile)
        {
            pawnProfile.thingDef = input.def;
            TestPawnElement(pawnProfile.thingDef);

            pawnProfile.trainabilityDef = input.RaceProps.trainability ?? null;
            TestPawnElement(pawnProfile.trainabilityDef, "TrainabilityDef");

            pawnProfile.intelligence = input.RaceProps.intelligence;
            TestPawnElement(pawnProfile.intelligence);

            pawnProfile.pawnKindDef = input.kindDef;
            TestPawnElement(pawnProfile.pawnKindDef);

            pawnProfile.royalTitleDef = input.royalty?.MostSeniorTitle?.def ?? null;
            TestPawnElement(pawnProfile.royalTitleDef, "RoyalTitleDef");

            pawnProfile.gender = input.gender;
            TestPawnElement(pawnProfile.gender);

            pawnProfile.ageBiological = input.ageTracker.AgeBiologicalTicks;
            TestPawnElement(pawnProfile.ageBiological, "AgeBiological");

            pawnProfile.ageChronological = input.ageTracker.AgeChronologicalTicks;
            TestPawnElement(pawnProfile.ageChronological, "AgeChronological");

            pawnProfile.factionDef = input.Faction?.def ?? null;
            TestPawnElement(pawnProfile.factionDef, "FactionDef");

            CheckFactionRelationKind(input, out FactionRelationKind inputPawnFactionRelationKind);
            pawnProfile.factionRelationKind = inputPawnFactionRelationKind;
            TestPawnElement(pawnProfile.factionRelationKind);

            CheckFactionOf(input, out string inputfactionOf);
            pawnProfile.factionOf = inputfactionOf;
            TestPawnElement(pawnProfile.factionOf, "FactionOf");

            CheckPawnIs(input, out string inputPawnIs);
            pawnProfile.pawnIs = inputPawnIs;
            TestPawnElement(pawnProfile.pawnIs, "PawnIS");

            //이하 조건은 아니지만 저장하는것
            pawnProfile.faction = input.Faction ?? null;
            pawnProfile.pawn = input;

            if (input.RaceProps.Humanlike)
            {
                pawnProfile.childhood = input.story.Childhood ?? null;
                pawnProfile.adulthood = input.story.Adulthood ?? null;
            }
            if (input.Spawned)
            {
                pawnProfile.position = input.Position;
                pawnProfile.map = input.Map;
            }
        }

        private void CheckPawnIs(Pawn pawn, out string pawnIs)
        {
            pawnIs = "IsNotColonistOrPrisonerOrSlave";
            if (pawn.IsFreeColonist)
            {
                pawnIs = "IsFreeColonist";
            }
            else if (pawn.IsColonist)
            {
                pawnIs = "IsColonist";
            }
            else if (pawn.IsPrisonerOfColony)
            {
                pawnIs = "IsPrisonerOfColony";
            }
            else if (pawn.IsPrisoner)
            {
                pawnIs = "IsPrisoner";
            }
            else if (pawn.IsSlaveOfColony)
            {
                pawnIs = "IsSlaveOfColony";
            }
            else if (pawn.IsSlave)
            {
                pawnIs = "IsSlave";
            }
        }

        private static void CheckFactionOf(Pawn pawn, out string factionOf)
        {
            factionOf = "OfUnkwon";
            if (pawn.Faction == Faction.OfPlayer)
            {
                factionOf = "OfPlayer";
            }
            else if (pawn.Faction == Faction.OfAncientsHostile)
            {
                factionOf = "OfAncientsHostile";
            }
            else if (pawn.Faction == Faction.OfAncients)
            {
                factionOf = "OfAncients";
            }
            else if (pawn.Faction == Faction.OfInsects)
            {
                factionOf = "OfInsects";
            }
            else if (pawn.Faction == Faction.OfMechanoids)
            {
                factionOf = "OfMechanoids";
            }
            else if (pawn.Faction == Faction.OfEmpire)
            {
                factionOf = "OfEmpire";
            }
            else if (pawn.Faction == Faction.OfPlayerSilentFail)
            {
                factionOf = "OfPlayerSilentFail";
            }
        }

        private static void CheckFactionRelationKind(Pawn pawn, out FactionRelationKind pawnFactionRelationKind)
        {
            if (pawn.Faction == null)
            {
                pawnFactionRelationKind = FactionRelationKind.Neutral;
            }
            else if (pawn.Faction != Faction.OfPlayer)
            {
                pawnFactionRelationKind = pawn.Faction.RelationWith(Faction.OfPlayer).kind;
            }
            else
            {
                pawnFactionRelationKind = FactionRelationKind.Neutral;
            }
        }

        private void TestPawnElement<T>(T pawnCondition, string nullDefString = null)
        {
            var pawnConditionDefString = nullDefString;
            if (nullDefString == null)
            {
                pawnConditionDefString = pawnCondition.GetType().ToString().Replace("Rimworld.", "").Replace("Verse.", "").Replace("System.", "");
            }
            var pawnConditionString = pawnCondition?.ToString() ?? "null";
            TestErrorLog($" * Pawn {pawnConditionDefString.First().ToString().ToUpper() + pawnConditionDefString.Substring(1)} : {pawnConditionString}");
        }

        private void TestErrorLog(params string[] arr1)
        {
            if (ChangeRace_Setting.ChangeRaceTestLog)
            {
                for (int i = 0; i < arr1.Length; i++)
                {
                    Log.Error(arr1[i]);
                }
            }
        }
    }
}
