﻿using RimWorld;
using System.Collections.Generic;
using UnityEngine;
using Verse;

namespace ChangeRace
{
    public class PawnProfile
    {
        public ThingDef thingDef;
        public TrainabilityDef trainabilityDef;
        public Intelligence intelligence;
        public PawnKindDef pawnKindDef;
        public RoyalTitleDef royalTitleDef;
        public Gender gender;
        public long ageBiological;
        public long ageChronological;
        public Faction faction;
        public FactionDef factionDef;
        public FactionRelationKind factionRelationKind;
        public string factionOf;
        public string pawnIs;
        public IntVec3 position;
        public Map map;
        public Pawn pawn;
        public BackstoryDef childhood;
        public BackstoryDef adulthood;
    }
    public class ChangeRace
    {
        public List<ConditionAndResult> conditionAndResult = new List<ConditionAndResult>();
    }

    public class FactionOf
    {
        public List<Condition> condition = new List<Condition>();
        public FactionResult factionResult = new FactionResult();
    }

    public class BackstoryLogic
    {
        public bool keepOriginalPawnBackstory = false;
        public List<string> backstoryElementSequence = new List<string>();
        public DefaultElementName defaultElementName = new DefaultElementName();
        internal DefaultElementName tempElementName = new DefaultElementName();
        public AlienRace.AlienBackstoryDef defaultBackstoryDef = new AlienRace.AlienBackstoryDef();
        public CustomElementName customElementName = new CustomElementName();
    }

    public class CustomElementName
    {
        public List<BackstoryElement<ThingDef>> prefixElement = new List<BackstoryElement<ThingDef>>();
        public List<BackstoryElement<ThingDef>> raceElement = new List<BackstoryElement<ThingDef>>();
        public List<BackstoryElement<string>> factionOf_BeforeElement = new List<BackstoryElement<string>>();
        public List<BackstoryElement<string>> factionOf_AfterElement = new List<BackstoryElement<string>>();
        public List<BackstoryElement<RoyalTitleDef>> royalTitleElement = new List<BackstoryElement<RoyalTitleDef>>();
        public List<BackstoryElement<FactionDef>> faction_BeforeElement = new List<BackstoryElement<FactionDef>>();
        public List<BackstoryElement<FactionDef>> faction_AfterElement = new List<BackstoryElement<FactionDef>>();
        public List<BackstoryElement<TrainabilityDef>> trainabilityElement = new List<BackstoryElement<TrainabilityDef>>();
        public List<BackstoryElement<Intelligence>> intelligenceElement = new List<BackstoryElement<Intelligence>>();
        public List<BackstoryElement<Gender>> gender_BeforeElement = new List<BackstoryElement<Gender>>();
        public List<BackstoryElement<Gender>> gender_AfterElement = new List<BackstoryElement<Gender>>();
        public List<BackstoryElement<Age>> ageElement = new List<BackstoryElement<Age>>();
        public List<BackstoryElement<PawnKindDef>> pawnKind_BeforeElement = new List<BackstoryElement<PawnKindDef>>();
        public List<BackstoryElement<PawnKindDef>> pawnKind_AfterElement = new List<BackstoryElement<PawnKindDef>>();
        public List<BackstoryElement<FactionRelationKind>> factionRelationKind_BeforeElement = new List<BackstoryElement<FactionRelationKind>>();
        public List<BackstoryElement<FactionRelationKind>> factionRelationKind_AfterElement = new List<BackstoryElement<FactionRelationKind>>();
        public List<BackstoryElement<string>> pawnIs_BeforeElement = new List<BackstoryElement<string>>();
        public List<BackstoryElement<string>> pawnIs_AfterElement = new List<BackstoryElement<string>>();
    }

    public class BackstoryElement<T>
    {
        public List<T> element;
        public bool nullElement = false;
        public bool keepOriginalBackstory = false;
        public bool humanlike = true;
        public bool nonHumanlike = true;
        public string elementName = "";
    }

    public class Age
    {
        public int max;
        public int min;
    }

    public class DefaultElementName
    {
        public string prefix = "";
        public string race = "";
        public string factionOf_Before = "";
        public string factionOf_After = "";
        public string royalTitle = "";
        public string faction_Before = "";
        public string faction_After = "";
        public string trainability = "";
        public string intelligence = "";
        public string gender_Before = "";
        public string gender_After = "";
        public string age = "";
        public string pawnKind_Before = "";
        public string pawnKind_After = "";
        public string factionRelationKind_Before = "";
        public string factionRelationKind_After = "";
        public string pawnIs_Before = "";
        public string pawnIs_After = "";
    }

    public class FactionResult
    {
        public bool keepOriginalFaction = false;
        public bool isPlayerFaction = false;
        public bool isWildMan = false;
    }

    public class ConditionAndResult
    {
        public List<Condition> condition = new List<Condition>();
        public List<ChangeRaceResult> changeRaceResult = new List<ChangeRaceResult>();
    }

    public class Condition
    {
        public List<ThingDef> thingDef = new List<ThingDef>();
        public List<TrainabilityDef> trainabilityDef = new List<TrainabilityDef>();
        public List<Intelligence> intelligence = new List<Intelligence>();
        public List<PawnKindDef> pawnKindDef = new List<PawnKindDef>();
        public List<string> pawnIs = new List<string>();
        public List<string> factionOf = new List<string>();
        public List<RoyalTitleDef> royalTitleDef = new List<RoyalTitleDef>();
        public List<Gender> gender = new List<Gender>();
        public List<FactionDef> factionDef = new List<FactionDef>();
        public bool forDefaultChangeRaceResult = false;
        public List<FactionRelationKind> factionRelationKind = new List<FactionRelationKind>();
    }

    public class ChangeRaceResult
    {
        public ThingDef thingDef = new ThingDef();
        public bool makeOfPlayer = false;
        public int chance = 100;
        public PawnKindDef pawnKindDef = new PawnKindDef();
        public PawnKindDef temporaryPawnKindDefForWildMan = new PawnKindDef();
        public List<HediffDef> addHediffDef = new List<HediffDef>();
        public KeepWhat keepWhat = new KeepWhat();
        public List<Appearance> appearance = new List<Appearance>();
        public int stack = 1;
        public GenderResult genderResult = new GenderResult();
        public bool unOfColonyPawnOrAnimalDropItem = false;
        public bool bondChangeToLover = true;
        public MakeFilth makeFilth = new MakeFilth();
        public bool useDefaultForbiddenTraits = true;
        public List<Trait> forbiddenTraits = new List<Trait>();
        public SoundDef soundDef;
        public SendMessageOrLetter sendMessageOrLetter = new SendMessageOrLetter();
    }

    public class SendMessageOrLetter
    {
        public string message = null;
        public MessageTypeDef messageTypeDef;
        //= MessageTypeDefOf.NeutralEvent;

        public string letterLabel = null;
        public string letterText = null;
        public LetterDef letterDef;
        //= LetterDefOf.NeutralEvent;
    }

    public class KeepWhat
    {
        public bool keepReration = false;
        public bool keepGender = false;
        public bool keepHairColor = false;
        public bool keepFaction = false;
        public bool keepThought_MemorySocial = false;
    }

    public class MakeFilth
    {
        public ThingDef filthDef;
        public int filthNum = 1;
    }

    public class GenderResult
    {
        public int Female = 0;
        public int Male = 0;
    }

    public class Appearance
    {
        public HeadTypeDef headTypeDef;
        public HairDef hairDef;
        public BodyTypeDef bodyTypeDef;
        public int chance = 100;
        public List<Color> hairColor;
    }
}