﻿using HarmonyLib;
using RimWorld;
using UnityEngine;
using Verse;

namespace MinchoWitch
{
    [StaticConstructorOnStartup]
    public static class Harmony_ApparelGraphicRecordGetter
    {
        static Harmony_ApparelGraphicRecordGetter()
        {
            new Harmony("MinchoWitch").Patch(AccessTools.Method(typeof(ApparelGraphicRecordGetter), "TryGetGraphicApparel", null, null), new HarmonyMethod(typeof(Harmony_ApparelGraphicRecordGetter).GetMethod("TryGetGraphicApparel_Prefix")), null, null, null);
            Log.Message("MinchoWitch_Patched");
        }

        public static bool TryGetGraphicApparel_Prefix(ref bool __result, Apparel apparel, ref BodyTypeDef bodyType, out ApparelGraphicRecord rec)
        {
            bool flag = false;
            foreach (var apparelComp in apparel.AllComps)
            {
                if (apparelComp is MinchoWitchHatSizeComp comp)
                {
                    flag = true;
                    break;
                }
            }
            if (!flag)
            {
                rec = default;
                return true;
            }
            else
            {
                if (bodyType == null)
                {
                    Log.Error("Getting apparel graphic with undefined body type.");
                    bodyType = BodyTypeDefOf.Male;
                }
                if (apparel.WornGraphicPath.NullOrEmpty())
                {
                    rec = new ApparelGraphicRecord(null, null);
                    return false;
                }

                string path = apparel.WornGraphicPath;

                if (apparel.def.apparel.LastLayer == ApparelLayerDefOf.Overhead || apparel.def.apparel.LastLayer == ApparelLayerDefOf.EyeCover || PawnRenderer.RenderAsPack(apparel) || apparel.WornGraphicPath == BaseContent.PlaceholderImagePath || apparel.WornGraphicPath == BaseContent.PlaceholderGearImagePath)
                {
                    foreach (var apparelComp in apparel.AllComps)
                    {
                        if (apparelComp is MinchoWitchHatSizeComp comp)
                        {
                            if (apparel.ParentHolder is Pawn_ApparelTracker pawn_ApparelTracker && pawn_ApparelTracker.pawn?.def == comp.Props.raceThingDef)
                            {
                                path = comp.Props.texturePath;
                                break;
                            }
                        }
                    }
                }
                else
                {
                    path = apparel.WornGraphicPath + "_" + bodyType.defName;

                    foreach (var apparelComp in apparel.AllComps)
                    {
                        if (apparelComp is MinchoWitchHatSizeComp comp)
                        {
                            if (apparel.ParentHolder is Pawn_ApparelTracker pawn_ApparelTracker && pawn_ApparelTracker.pawn?.def == comp.Props.raceThingDef)
                            {
                                path = comp.Props.texturePath + "_" + bodyType.defName;
                            }
                        }
                    }
                }
                Shader shader = ShaderDatabase.Cutout;
                if (apparel.def.apparel.useWornGraphicMask)
                {
                    shader = ShaderDatabase.CutoutComplex;
                }

                Graphic graphic = GraphicDatabase.Get<Graphic_Multi>(path, shader, apparel.def.graphicData.drawSize, apparel.DrawColor);
                rec = new ApparelGraphicRecord(graphic, apparel);
                __result = true;

                return false;
            }
        }
    }
}
